/*
 * Copyright 2011 Jan Ouwens
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.jqno.equalsverifier.testhelpers.types;

import static nl.jqno.equalsverifier.testhelpers.Util.nullSafeHashCode;

public final class MutableCanEqualColorPoint extends ImmutableCanEqualPoint {
	private Color color;
	
	public MutableCanEqualColorPoint(int x, int y, Color color) {
		super(x, y);
		this.color = color;
	}
	
	@Override
	public boolean canEqual(Object obj) {
		return obj instanceof MutableCanEqualColorPoint;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof MutableCanEqualColorPoint)) {
			return false;
		}
		MutableCanEqualColorPoint p = (MutableCanEqualColorPoint)obj;
		return p.canEqual(this) && super.equals(p) && color == p.color;
	}
	
	@Override
	public int hashCode() {
		return nullSafeHashCode(color) + (31 * super.hashCode());
	}
	
	@Override
	public String toString() {
		return super.toString() + "," + color;
	}
}
